#ifndef __PLANE_H
#define __PLANE_H

#include "vector.h"

class CPlane
{
public:
     CVector N;     
     double  D;      

    
 
     CPlane(float a = 1, float b = 0, float c = 0, float d = 0)
     {
          N = CVector(a, b, c);
          D = d;
     }
 
     CPlane(const CVector& normal, double d = 0)
     {
          N = normal;
          D = d;
     }

    
     CPlane(const CPlane& plane)
     {
          N = plane.N;
          D = plane.D;
     }

   
     CPlane(const CVector& vertexA, const CVector& vertexB, const CVector& vertexC)
     {
          CVector normalA((vertexC - vertexA) | 1.0);  
          CVector normalB((vertexC - vertexB) | 1.0);   

          N = (normalA ^ normalB) | 1.0;    
          D = -vertexA % N;                
     }

  
     const CPlane& operator=(const CPlane& plane)
     {    
          N = plane.N;
          D = plane.D;

          return *this;
     }

      
     const bool operator==(const CPlane& plane) const
     {    
          return N == plane.N && D == plane.D;
     }

 
     const bool operator!=(const CPlane& plane) const
     {
          return !(*this == plane);
     }

   
     const bool inline PointOnPlane(const CVector& point) const
     {
          return DistanceToPlane(point) == 0;
     }

 
     const double inline DistanceToPlane(const CVector& point) const
     {
          return N % point + D;
     }

    
};

#endif    //__PLANE_H