bool InitDirectXAudio(HWND hwnd)
{
	char pathStr[MAX_PATH];		// path for audio file
	WCHAR wcharStr[MAX_PATH];

	// create the loader object
	if (FAILED(CoCreateInstance(CLSID_DirectMusicLoader, NULL, CLSCTX_INPROC,
						   IID_IDirectMusicLoader8, (void**)&dmusicLoader)))
	{
		MessageBox(hwnd, "Unable to create the IDirectMusicLoader8 object!\nPress OK to exit",
				 "ERROR!", MB_OK);
		return false;
	}

	// create the performance object
	if (FAILED(CoCreateInstance(CLSID_DirectMusicPerformance, NULL, CLSCTX_INPROC,
						   IID_IDirectMusicPerformance8, (void**)&dmusicPerformance)))
	{
		MessageBox(hwnd, "Unable to create the IDirectMusicPerformance8 object!\nPress OK to exit",
				 "ERROR!", MB_OK);
		return false;
	}

	// initialize the performance with the standard audio path
	dmusicPerformance->InitAudio(NULL, NULL, hwnd, DMUS_APATH_SHARED_STEREOPLUSREVERB, 64,
						    DMUS_AUDIOF_ALL, NULL);

	// retrieve the current directory
	GetCurrentDirectory(MAX_PATH, pathStr);

	// convert to unicode string
	MultiByteToWideChar(CP_ACP, 0, pathStr, -1, wcharStr, MAX_PATH);

	// set the search directory
	dmusicLoader->SetSearchDirectory(GUID_DirectMusicAllTypes, wcharStr, FALSE);

	return true;
}

// LoadSegment()
// desc: load a segment from a file
bool LoadSegment(HWND hwnd, char *filename)
{
	WCHAR wcharStr[MAX_PATH];

	// convert filename to unicode string
	MultiByteToWideChar(CP_ACP, 0, filename, -1, wcharStr, MAX_PATH);

	// load the segment from file
	if (FAILED(dmusicLoader->LoadObjectFromFile(CLSID_DirectMusicSegment,
									    IID_IDirectMusicSegment8,
									    wcharStr,
									    (void**)&dmusicSegment)))
	{
		MessageBox(hwnd, "Audio file not found! Press OK to exit",
				 "ERROR!", MB_OK);

		return false;
	}

	// download the segment's instruments to the synthesizer
	dmusicSegment->Download(dmusicPerformance);

	return true;
}

// PlaySegment()
// desc: start playing a segment
void PlaySegment(IDirectMusicPerformance8* dmPerf, IDirectMusicSegment8* dmSeg)
{
	// play the segment
	dmPerf->PlaySegmentEx(dmSeg, NULL, NULL, 0, 0, NULL, NULL, NULL);
}

// StopSegment()
// desc: stop a segment from playing
void StopSegment(IDirectMusicPerformance8* dmPerf, IDirectMusicSegment8* dmSeg)
{
	// stop the dmSeg from playing
	dmPerf->StopEx(dmSeg, 0, 0);
}

// CloseDown()
// desc: shutdown music performance
void CloseDown(IDirectMusicPerformance8* dmPerf)
{
	// stop the music
	dmPerf->Stop(NULL, NULL, 0, 0);

	// close down DirectMusic
	dmPerf->CloseDown();
}
