#ifndef __MD2_H
#define __MD2_H

#include "textures.h"

#define MAX_FRAMES 512


typedef struct
{
   float point[3];
} vector_t;

vector_t operator-(vector_t a, vector_t b)
{
	vector_t c;

	c.point[0] = a.point[0] - b.point[0];
	c.point[1] = a.point[1] - b.point[1];
	c.point[2] = a.point[2] - b.point[2];

	return c;
}

vector_t operator*(float f, vector_t b)
{
	vector_t c;

	c.point[0] = f * b.point[0];
	c.point[1] = f * b.point[1];
	c.point[2] = f * b.point[2];

	return c;
}


vector_t operator/(vector_t a, vector_t b)
{
	vector_t c;

	c.point[0] = a.point[0] / b.point[0];
	c.point[1] = a.point[1] / b.point[1];
	c.point[2] = a.point[2] / b.point[2];

	return c;
}


vector_t operator+(vector_t a, vector_t b)
{
	vector_t c;

	c.point[0] = a.point[0] + b.point[0];
	c.point[1] = a.point[1] + b.point[1];
	c.point[2] = a.point[2] + b.point[2];

	return c;
}


typedef struct
{
   float s;
   float t;
} texCoord_t;


typedef struct
{
   short s;
   short t;
} stIndex_t;


typedef struct
{
   unsigned char v[3];
   unsigned char normalIndex;
} framePoint_t;

typedef struct
{
   float scale[3];
   float translate[3];
   char name[16];
   framePoint_t fp[1];
} frame_t;


typedef struct
{
   unsigned short meshIndex[3];		
   unsigned short stIndex[3];	
} mesh_t;

// the model data
typedef struct
{
	int numFrames;			
	int numPoints;			
	int numTriangles;		
    int numST;				
	int frameSize;			
	int texWidth, texHeight;	
	int currentFrame;		
	int nextFrame;				
	float interpol;			
	mesh_t *triIndex;		
	texCoord_t *st;				
	vector_t *pointList;		
	texture_t *modelTex;	
} modelData_t;

typedef struct
{
   int ident;	
   int version;		
   int skinwidth;   
   int skinheight;   
   int framesize;    
   int numSkins;     
   int numXYZ;       
   int numST;       
   int numTris;     
   int numGLcmds;
   int numFrames;    
   int offsetSkins; 
   int offsetST;     
   int offsetTris;   
   int offsetFrames; 
   int offsetGLcmds; 
   int offsetEnd;   
} modelHeader_t;

#endif