#ifndef __VECTOR_H
#define __VECTOR_H

#include <math.h>

 
typedef float scalar_t;

class CVector
{
public:
     scalar_t x;
     scalar_t y;
     scalar_t z;     
public:
     CVector(scalar_t a = 0, scalar_t b = 0, scalar_t c = 0) : x(a), y(b), z(c) {}
     CVector(const CVector &vec) : x(vec.x), y(vec.y), z(vec.z) {}

      
     const CVector &operator=(const CVector &vec)
     {
          x = vec.x;
          y = vec.y;
          z = vec.z;

          return *this;
     }

 
     const bool operator==(const CVector &vec) const
     {
          return ((x == vec.x) && (y == vec.y) && (z == vec.z));
     }

 
     const bool operator!=(const CVector &vec) const
     {
          return !(*this == vec);
     }

     
     const CVector operator+(const CVector &vec) const
     {
          return CVector(x + vec.x, y + vec.y, z + vec.z);
     }

 
     const CVector operator+() const
     {    
          return CVector(*this);
     }

 
     const CVector& operator+=(const CVector& vec)
     {    x += vec.x;
          y += vec.y;
          z += vec.z;
          return *this;
     }
 
     const CVector operator-(const CVector& vec) const
     {    
          return CVector(x - vec.x, y - vec.y, z - vec.z);
     }
     
 
     const CVector operator-() const
     {    
          return CVector(-x, -y, -z);
     }

 
     const CVector &operator-=(const CVector& vec)
     {
          x -= vec.x;
          y -= vec.y;
          z -= vec.z;

          return *this;
     }

   
     const CVector &operator*=(const scalar_t &s)
     {
          x *= s;
          y *= s;
          z *= s;
          
          return *this;
     }

 
     const CVector &operator/=(const scalar_t &s)
     {
          const float recip = 1/s;  

          x *= recip;
          y *= recip;
          z *= recip;

          return *this;
     }

 
     const CVector operator*(const scalar_t &s) const
     {
          return CVector(x*s, y*s, z*s);
     }
  
     friend inline const CVector operator*(const scalar_t &s, const CVector &vec)
     {
          return vec*s;
     }

 
     const CVector operator/(scalar_t s) const
     {
          s = 1/s;

          return CVector(s*x, s*y, s*z);
     }
 
     const CVector CrossProduct(const CVector &vec) const
     {
          return CVector(y*vec.z - z*vec.y, z*vec.x - x*vec.z, x*vec.y - y*vec.x);
     }

  
     const CVector operator^(const CVector &vec) const
     {
          return CVector(y*vec.z - z*vec.y, z*vec.x - x*vec.z, x*vec.y - y*vec.x);
     }
 
     const scalar_t DotProduct(const CVector &vec) const
     {
          return x*vec.x + y*vec.x + z*vec.z;
     }
 
     const scalar_t operator%(const CVector &vec) const
     {
          return x*vec.x + y*vec.x + z*vec.z;
     }

 
     const scalar_t Length() const
     {
          return (scalar_t)sqrt((double)(x*x + y*y + z*z));
     }

   
     const CVector UnitVector() const
     {
          return (*this) / Length();
     }
 
     void Normalize()
     {
          (*this) /= Length();
     }

     const scalar_t operator!() const
     {
          return sqrtf(x*x + y*y + z*z);
     }

 
     const CVector operator | (const scalar_t length) const
     {
          return *this * (length / !(*this));
     }

   
     const CVector& operator |= (const float length)
     {
          return *this = *this | length;
     }

  
     const float inline Angle(const CVector& normal) const
     {
          return acosf(*this % normal);
     }

     
     const CVector inline Reflection(const CVector& normal) const
     {    
          const CVector vec(*this | 1);      
          return (vec - normal * 2.0 * (vec % normal)) * !*this;
     }

};

#endif